from django.db import models
from wagtail.core import blocks
from django.utils.text import camel_case_to_spaces
from wagtail.images.blocks import ImageChooserBlock

from weo import mixins

# Image en tête
# -------------


class ImageEnTete(models.ForeignKey):
    def __init__(self, **kwargs):
        kwargs['to'] = 'wagtailimages.Image'
        kwargs['null'] = True
        kwargs['blank'] = True
        kwargs['on_delete'] = models.SET_NULL
        kwargs['related_name'] = '+'
        kwargs['verbose_name'] = 'image en tête'
        kwargs['help_text'] = ("Utilisée pour mettre en valeur l'article dans "
                               "des liens, via twitter, etc.")
        super().__init__(**kwargs)


# Contenus
# ========

contenus = {}


def register_contenu():
    """
    Macro hygiénique pour conserver une liste des blocs décrits,
    groupés par type.
    """
    class Wrapper(object):

        def __init__(self, cls, *args):
            self.wrapped = cls(*args)
            title = camel_case_to_spaces(cls.__name__).replace(' ', '_')
            group = cls._meta_class.group
            if group not in contenus:
                contenus[group] = []
            contenus[group].append((title, cls),)

        def __call__(self):
            return self.wrapped

    return Wrapper


# Contenu terminal
# ----------------


# Contenu texte
# ^^^^^^^^^^^^^

class InlineRichTextBlock(blocks.RichTextBlock):
    def __init__(self, *args, **kwargs):
        kwargs['features'] = ['bold', 'italic', 'link', 'document-link']
        return super().__init__(*args, **kwargs)


@register_contenu()
class Titre(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'title'
        group = 'texte'

    niveau = blocks.ChoiceBlock(
            default='h3',
            choices=[
                ('h{}'.format(i), 'Titre de niveau {}'.format(i))
                for i in range(3, 7)
            ],
            icon='title',
    )
    texte = InlineRichTextBlock(icon='pilcrow')


@register_contenu()
class Exergue(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'form'
        group = 'texte'

    texte = blocks.RichTextBlock(icon='pilcrow')


@register_contenu()
class Paragraphe(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'pilcrow'
        group = 'texte'

    texte = blocks.RichTextBlock(icon='pilcrow')


@register_contenu()
class ChiffreCle(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'cogs'
        group = 'texte'
        label = "chiffre clé"

    chiffre = blocks.IntegerBlock(icon='code')


@register_contenu()
class Alerte(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'warning',
        group = 'texte'

    texte = blocks.RichTextBlock(icon='pilcrow')
    classe = blocks.ChoiceBlock(
            default='primary',
            choices=[
                ('primary', 'Primaire'),
                ('secondary', 'Secondaire'),
                ('success', 'Succès'),
                ('warning', 'Attention'),
                ('danger', 'Danger'),
                ('info', 'Info'),
                ('light', 'claire'),
                ('dark', 'sombre'),
            ],
    )


@register_contenu()
class Bouton(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'pick'
        group = 'texte'

    texte = blocks.TextBlock(icon='pilcrow')
    lien = blocks.PageChooserBlock(icon='link')


def contenus_textes():
    return [(nom, bloc()) for nom, bloc in contenus['texte']]


# Contenu media
# ^^^^^^^^^^^^^

@register_contenu()
class Image(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'image'
        group = 'media'

    image = ImageChooserBlock(icon='image')


def contenus_medias():
    return [(nom, bloc()) for nom, bloc in contenus['media']]

# Contenu interne
# ^^^^^^^^^^^^^^^


@register_contenu()
class TableMatieres(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'list-ul'
        group = 'interne'
        label = "table des matières"

    coulissant = blocks.BooleanBlock(
            label="coulissant avec le défilement",
            required=False,
    )
    enfants = blocks.BooleanBlock(
            label="inclut les liens vers les pages enfants",
    )


@register_contenu()
class LiensEnfants(mixins.SetBlockTheme, blocks.StaticBlock):
    class Meta:
        icon = 'link'
        group = 'interne'
        label = "pages enfants"


@register_contenu()
class Espace(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'site'
        group = 'interne'
        label = "lien vers un espace"

    page = blocks.PageChooserBlock(target_model='wss.Espace')


@register_contenu()
class Page(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'snippet'
        group = 'interne'
        label = "lien vers une page"

    page = blocks.PageChooserBlock(target_model='wss.SitePage')


@register_contenu()
class Actualite(mixins.SetBlockTheme, blocks.StaticBlock):
    class Meta:
        icon = 'time'
        group = 'interne'
        label = "dernière actualité"


@register_contenu()
class Evenement(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'date'
        group = 'interne'
        label = "prochain évènement"

    apres = blocks.IntegerBlock(
            default=3,
            help_text="Évènement après dans n jours.",
    )
    avant = blocks.IntegerBlock(
            default=30,
            help_text="Évènement avant dans n jours.",
    )


@register_contenu()
class Formation(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'date'
        group = 'interne'
        label = "prochaine formation"

    apres = blocks.IntegerBlock(
            default=3,
            help_text="Formation après dans n jours.",
    )
    avant = blocks.IntegerBlock(
            default=30,
            help_text="Formation avant dans n jours.",
    )


def contenus_internes():
    return [(nom, bloc()) for nom, bloc in contenus['interne']]


# Contenu layout
# --------------

# Contenu terminal
# ^^^^^^^^^^^^^^^^


def contenus_terminaux():
    return contenus_textes() + contenus_medias() + contenus_internes()


# Contenu pour 2 colonnes
# ^^^^^^^^^^^^^^^^^^^^^^^


class OneOneCol(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'date'
        group = 'disposition'
        label = "deux colonnes de largeur 1"

    gauche = blocks.StreamBlock(contenus_terminaux())
    droite = blocks.StreamBlock(contenus_terminaux())


def layout_2col():
    return [('colonne_1+1', OneOneCol())]


def contenu_2col():
    return contenus_terminaux() + layout_2col()


# Contenu pour 3 colonnes
# ^^^^^^^^^^^^^^^^^^^^^^^


class OneTwoCol(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'grip'
        group = 'disposition'
        label = "deux colonnes de largeur 1 | 2"

    gauche = blocks.StreamBlock(contenus_terminaux())
    droite = blocks.StreamBlock(contenu_2col())


class TwoOneCol(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'grip'
        group = 'disposition'
        label = "deux colonnes de largeur 2 | 1"

    gauche = blocks.StreamBlock(contenu_2col())
    droite = blocks.StreamBlock(contenus_terminaux())


class OneOneOneCol(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'grip'
        group = 'disposition'
        label = "trois colonnes de largeur 1"

    gauche = blocks.StreamBlock(contenus_terminaux())
    centre = blocks.StreamBlock(contenus_terminaux())
    droite = blocks.StreamBlock(contenus_terminaux())


def layout_3col():
    return [
            ('colonne_1+2', OneTwoCol()),
            ('colonne_2+1', TwoOneCol()),
            ('colonne_1+1+1', OneOneOneCol()),
    ]


def contenu_3col():
    return contenus_terminaux() + layout_3col()


# Contenu pour 4 colonnes
# ^^^^^^^^^^^^^^^^^^^^^^^


class OneThrCol(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'grip'
        group = 'disposition'
        label = "deux colonnes de largeur 1 | 3"

    gauche = blocks.StreamBlock(contenus_terminaux())
    droite = blocks.StreamBlock(contenu_3col())


class ThrOneCol(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'grip'
        group = 'disposition'
        label = "deux colonnes de largeur 3 | 1"

    gauche = blocks.StreamBlock(contenu_3col())
    droite = blocks.StreamBlock(contenus_terminaux())


class TwoTwoCol(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'grip'
        group = 'disposition'
        label = "deux colonnes de largeur 2"

    gauche = blocks.StreamBlock(contenu_2col())
    droite = blocks.StreamBlock(contenu_2col())


class OneTwoOneCol(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'grip'
        group = 'disposition'
        label = "trois colonnes de largeur 1 | 2 | 1"

    gauche = blocks.StreamBlock(contenus_terminaux())
    centre = blocks.StreamBlock(contenu_2col())
    droite = blocks.StreamBlock(contenus_terminaux())


class OneOneOneOneCol(mixins.SetBlockTheme, blocks.StructBlock):
    class Meta:
        icon = 'grip'
        group = 'disposition'
        label = "quatre colonnes de largeur 1"

    gauche = blocks.StreamBlock(contenus_terminaux())
    centre_gauche = blocks.StreamBlock(contenus_terminaux())
    centre_droit = blocks.StreamBlock(contenus_terminaux())
    droite = blocks.StreamBlock(contenus_terminaux())


def layout_4col():
    return [
            ('colonne_1+3', OneThrCol()),
            ('colonne_3+1', ThrOneCol()),
            ('colonne_2+2', TwoTwoCol()),
            ('colonne_1+2+1', OneTwoOneCol()),
            ('colonne_1+1+1+1', OneOneOneOneCol()),
    ]


def contenu_4col():
    return contenus_terminaux() + layout_4col()
