from django.core import validators
from django.db import models
from modelcluster.contrib.taggit import ClusterTaggableManager
from modelcluster.fields import ParentalKey, ParentalManyToManyField
from modelcluster.models import ClusterableModel
from wagtail.admin.edit_handlers import (
        FieldPanel, MultiFieldPanel, InlinePanel, StreamFieldPanel,
        PageChooserPanel, FieldRowPanel
)
from wagtail.core import blocks
from wagtail.core.fields import RichTextField, StreamField
from wagtail.core.models import Page
from wagtail.images.edit_handlers import ImageChooserPanel

import weo.models
from wss import mixins, taxonomy


class SitePage(mixins.SetSiteTheme, Page):
    """
    Elle sert à publier du contenu statique non périssable : documentation,
    argumentaires, etc.  La page générique est la page de base du site web.
    Elle représentera néanmoins le type de page le plus utilisé avec le type
    actualite et le type evenement.
    """

    class Meta:
        verbose_name = "page"

    # Contraintes de structure
    # ------------------------

    parent_page_types = ['Espace', 'Accueil', 'SitePage']
    subpage_types = ['SitePage']

    # Contenu
    # -------

    image_en_tete = weo.models.ImageEnTete()

    contenu = StreamField(blocks.StreamBlock(weo.models.contenu_2col()))

    content_panels = Page.content_panels + [
            ImageChooserPanel('image_en_tete'),
            StreamFieldPanel('contenu'),
            ]

    # Promotion
    # ---------

    description_seo = models.TextField(
            "description SEO",
            help_text="Description courte de pour les moteurs de recherche.",
            blank=True,
            )

    mots_cles = ClusterTaggableManager(
            "Mots clés",
            through='MotsClesTag',
            blank=True,
            related_name='tag',
            help_text="Une liste de mots clés séparés par des virgules.",
            )

    technicite = models.PositiveSmallIntegerField(
            "Technicité",
            null=True,
            blank=True,
            validators=[validators.MaxValueValidator(5)],
            help_text="Un score de 0 à 5.",
            )

    production = ClusterTaggableManager(
            "Type de production",
            through='ProductionTag',
            blank=True,
            related_name='prod',
            help_text="Une liste de types de productions, séparés par des "
            "virgules.",
            )

    promote_panels = [
            MultiFieldPanel([
                FieldPanel('slug'),
                FieldPanel('seo_title'),
                FieldPanel('description_seo'),
                FieldPanel('mots_cles'),
            ], "Configuration SEO"),

            MultiFieldPanel([
                InlinePanel(
                    'pages_liees_from',
                    label="Page(s) liée(s)",
                    panels=[PageChooserPanel('pages_liees_to')],
                    ),
            ], "Pages liées"),
            MultiFieldPanel([
                FieldPanel('technicite'),
                FieldPanel('production'),
            ], "Attributs métiers"),
    ]

    # Réglages
    # --------

    disp_mots_cles = models.BooleanField(
            "Afficher les mots clés",
            default=True,
            help_text="Fait apparaître les mots clés dans la page concernée.",
            )

    settings_panels = Page.settings_panels + [
            MultiFieldPanel([
                FieldPanel('disp_mots_cles'),
            ], "Options d'affichage"),
    ]

    # Pages liées
    # ----------

    @property
    def pages_liees(self):
        return [
                related.pages_liees_to
                for related in self.pages_liees_from.all()
        ] + [
                related.pages_liees_from
                for related in taxonomy.PagesLiees.objects.all()
                if related.pages_liees_to == self
        ]


class ActualiteIndex(mixins.SiteCommonIndex, SitePage):
    """
    Elle sert à recueillir/structurer les actualités pour entretenir les
    espaces en silos. Des actualités « hors espace » (transverses à tout le
    site) sont possibles via une listes des actualités sous la racine.
    """
    class Meta:
        verbose_name = "liste des actualités"

    parent_page_types = ['Accueil', 'Espace']
    subpage_types = ['Actualite']

    # Reglages
    # --------

    disp_rubriques = models.BooleanField(
            "Afficher les rubriques",
            default=True,
            help_text="Fait apparaître les rubriques dans la page concernée.",
            )

    filter_rubriques = ParentalManyToManyField(
            'Rubrique',
            verbose_name="Filtrage sur les rubriques",
            help_text=("Tout est inclut par défaut. Maintenez [Ctrl] pour en "
                       "sélectionner ou désélectionner plusieurs."),
            related_name='filter_rubriques',
            blank=True,
            )

    settings_panels = Page.settings_panels + [
            MultiFieldPanel([
                FieldPanel('disp_mots_cles'),
                FieldPanel('disp_rubriques'),
                FieldPanel('disp_pleine_page'),
                FieldPanel('disp_silo'),
            ], "Options d'affichage"),
            MultiFieldPanel([
                FieldPanel('filter_motscles'),
                FieldPanel('filter_productions'),
                FieldPanel('filter_territoires'),
                FieldPanel('filter_rubriques'),
                FieldPanel('filter_technicite'),
            ], "Options de sélection"),
    ]


class Actualite(SitePage):
    """
    Elle sert à publier du contenu d’actualité périssable : annonces,
    réactions, relais d’information, etc.
    """
    class Meta:
        verbose_name = "actualité"

    # Contraintes de structure
    # ------------------------

    parent_page_types = ['ActualiteIndex']
    subpage_types = []

    # Contenu
    # -------

    rubrique = ParentalKey('Rubrique', on_delete=models.PROTECT)

    territoire = ParentalKey(
            'Territoire',
            on_delete=models.PROTECT,
            blank=True,
            null=True,
            help_text="Territoire concerné. Toute la région par défaut.",
            )

    resume = RichTextField(
            "Résumé",
            blank=True,
            help_text="Résumé de l'article pour affichage compact.",
            )

    content_panels = Page.content_panels + [
            FieldPanel('rubrique'),
            FieldPanel('territoire'),
            FieldPanel('resume'),
            ImageChooserPanel('image_en_tete'),
            StreamFieldPanel('contenu'),
            ]

    # Réglages
    # --------

    disp_resume = models.BooleanField(
            "Afficher le résumé",
            default=True,
            help_text="Affiche le résumé en mode « pleine page ».",
            )

    epingle = models.BooleanField(
            "Épingler en haut des listes",
            default=False,
            help_text="Met en valeur l'article en tête des listes.",
            )

    settings_panels = Page.settings_panels + [
            MultiFieldPanel([
                FieldPanel('disp_mots_cles'),
                FieldPanel('disp_resume'),
                FieldPanel('epingle'),
            ], "Options d'affichage"),
    ]


class EvenementIndex(mixins.SiteCommonIndex, SitePage):
    """
    Elle sert à lister les évènements, plutôt pour avoir une vue
    complète/exhaustive. Les vues partielles (filtrées) seront plutôt destinées
    à être embarqués dans des blocs.
    """
    class Meta:
        verbose_name = "liste des évènements"

    parent_page_types = ['Accueil', 'Espace']
    subpage_types = ['Evenement']

    settings_panels = Page.settings_panels + [
            MultiFieldPanel([
                FieldPanel('disp_mots_cles'),
                FieldPanel('disp_pleine_page'),
                FieldPanel('disp_silo'),
            ], "Options d'affichage"),
            MultiFieldPanel([
                FieldPanel('filter_motscles'),
                FieldPanel('filter_productions'),
                FieldPanel('filter_territoires'),
                FieldPanel('filter_technicite'),
            ], "Options de sélection"),
    ]


class EvtRange(ClusterableModel):
    debut = models.DateTimeField('début')
    fin = models.TimeField()
    evenement = ParentalKey('Evenement', related_name='ranges')


class Evenement(SitePage):
    """
    Elle sert à publier un évènement temporellement défini : évènement,
    formation, campagne, etc.
    """
    class Meta:
        verbose_name = "évènement"

    # Contraintes de structure
    # ------------------------

    parent_page_types = ['EvenementIndex']
    subpage_types = []

    # Contenu
    # -------

    territoire = ParentalKey(
            'Territoire',
            on_delete=models.PROTECT,
            blank=True,
            null=True,
            help_text="Territoire concerné. Toute la région par défaut.",
            )

    localisation = models.CharField(
            max_length=255,
            blank=True,
            help_text="Exemples : Amiens, Maison de la culture ; "
                      "Lille, Grand Palais",
            )

    geolocalisation = models.CharField(
            max_length=255,
            blank=True,
            validators=[validators.RegexValidator(r'\d+\.\d+,\d+\.\d+')],
            help_text="Exemple : 50.4839,2.1338"
            )

    content_panels = Page.content_panels + [
            MultiFieldPanel([
                InlinePanel(
                    'ranges',
                    label="Plage horaire",
                    min_num=1,
                    panels=[FieldRowPanel([
                        FieldPanel('debut'),
                        FieldPanel('fin'),
                        ])],
                    ),
            ], "Plage(s) horaire(s)"),
            MultiFieldPanel([
                FieldPanel('territoire'),
                FieldPanel('localisation'),
                FieldPanel('geolocalisation'),
            ], "Localisation"),
            ImageChooserPanel('image_en_tete'),
            StreamFieldPanel('contenu'),
            ]

    # Réglages
    # --------

    epingle = models.BooleanField(
            "Épingler en haut des listes",
            default=False,
            help_text="Met en valeur l'article en tête des listes.",
            )

    settings_panels = Page.settings_panels + [
            MultiFieldPanel([
            ], "Options d'affichage"),
    ]


class Formation(Evenement):
    """
    Une formation est un évènement géré séparément.
    """

    # Contraintes de structure
    # ------------------------

    parent_page_types = ['FormationIndex']
    subpage_types = []


class FormationIndex(mixins.SiteCommonIndex, SitePage):
    """
    Elle sert à lister les formations, plutôt pour avoir une vue
    complète/exhaustive. Les vues partielles (filtrées) seront plutôt destinées
    à être embarqués dans des blocs.
    """
    class Meta:
        verbose_name = "liste des formations"

    parent_page_types = ['Accueil', 'Espace']
    subpage_types = ['Formation']


class Espace(SitePage):
    """
    La page d’espace est le premier niveau sous la racine du site. Ça permet de
    constituer le site en silo, par exemple définis selon le type de visiteur.
    Les détails de ses attributs varieront en fonction des besoins graphiques
    et ergonomiques.
    """

    # Contraintes de structure
    # ------------------------

    parent_page_types = ['Espace', 'Accueil', 'SitePage']
    subpage_types = ['SitePage', 'ActualiteIndex', 'FormationIndex']

    slogan = models.CharField(
            max_length=255,
            blank=True,
            help_text="Accroche courte",
    )

    contenu4 = StreamField(
            blocks.StreamBlock(weo.models.contenu_4col()),
            verbose_name="contenu 4 colonnes",
            )

    content_panels = Page.content_panels + [
            FieldPanel('slogan'),
            ImageChooserPanel('image_en_tete'),
            StreamFieldPanel('contenu4'),
            ]


class Accueil(mixins.RootPage, SitePage):
    """
    La page d’accueil est la page racine du site. On arrive dessus par défaut.
    Les détails de ses attributs varieront en fonction des besoins graphiques
    et ergonomiques.
    """

    # Contraintes de structure
    # ------------------------

    parent_page_types = [Page]  # root page is of type Page and isn't creatable
    subpage_types = [
            'Espace',
            'SitePage',
            'ActualiteIndex',
            'EvenementIndex',
            'FormationIndex',
    ]

    contenu4 = StreamField(
            blocks.StreamBlock(weo.models.contenu_4col()),
            verbose_name="contenu 4 colonnes",
            )

    content_panels = Page.content_panels + [
            ImageChooserPanel('image_en_tete'),
            StreamFieldPanel('contenu4'),
            ]
